@@ -1,7 +1,9 @@
 
-__version__ = "5.8"
+__version__ = "6.00"
+__author__ = "Alex Al-Saffar"
+__date__ = "2022-03-16"
 docs = r"https://crocodile.readthedocs.io/en/latest/"
 git = r"https://github.com/thisismygitrepo/crocodile"
 release_notes = """
-Bug fixes
+concice code version.
 """
