@@ -3,7 +3,7 @@
 from crocodile.toolbox import *
 from crocodile import __version__
 
-croc = P(__file__).parent.joinpath("art/happy_croco").read_text()
+croc = P(__file__).parent.joinpath("art").search().sample()[0].read_text()
 print(f"Crocodile Shell {__version__}")
 print(f"Made with ❤️")
 print(croc)
