@@ -2,20 +2,24 @@
 
 import crocodile.toolbox as tb
 import platform
+import os
+
 
 system = platform.system()
-sep = ";" if system == "Windows" else ":"
+OS = os.getenv("OS")  # Windows_NT
+sep = ";" if system == "Windows" else ":"  # path separator, not to be confused with P.sep
 
-os = tb.os
 P = tb.P
 L = tb.List
 
 tm = tb.Terminal()
+
+# ============================== Common Paths ============================
 DotFiles = P.home().joinpath("dotfiles")
 
 LocalAppData = P(tmp) if (tmp := os.getenv("LOCALAPPDATA")) else None  # C:\Users\username\AppData\Local
 AppData = P(tmp) if (tmp := os.getenv("APPDATA")) else None  # C:\Users\username\AppData\Roaming
-WindowsApps = tmp if (tmp := P(r"C:\Users\Alex\AppData\Local\Microsoft\WindowsApps")) else None
+WindowsApps = AppData.joinpath(r"Microsoft\WindowsApps") if AppData else None  # this path is already in PATH. Thus, useful to add symlinks and shortcuts to apps that one would like to be in the PATH.
 
 ProgramData = P(tmp) if (tmp := os.getenv("PROGRAMDATA")) else None  # C:\ProgramData
 ProgramFiles = P(tmp) if (tmp := os.getenv("ProgramFiles")) else None  # C:\Program Files
@@ -32,53 +36,46 @@ Temp = Tmp
 Path = L(os.getenv("PATH").split(sep)).apply(P)
 PSPath = L(tmp.split(sep)).apply(P) if (tmp := os.getenv("PSModulePath")) else None
 
-HostName = os.getenv("COMPUTERNAME")
-UserDomain = os.getenv("USERDOMAIN")
-UserName = os.getenv("USERNAME")
-OS = os.getenv("OS")  # Windows_NT
-Public = P(tmp) if (tmp := os.getenv("PUBLIC")) else None
+HostName = os.getenv("COMPUTERNAME")  # e.g. "MY-SURFACE"
+LogonServer = os.getenv("LOGONSERVER")  # e.g. "\\MY-SURFACE"
+UserDomain = os.getenv("USERDOMAIN")  # e.g. HAD OR MY-SURFACE
+UserDomainRoaming = P(tmp) if (tmp := os.getenv("USERDOMAIN_ROAMINGPROFILE")) else None  # e.g. SURFACE
+UserName = os.getenv("USERNAME")  # e.g: alex
+UserProfile = P(tmp) if (tmp := os.getenv("USERPROFILE")) else None  # e.g C:\Users\eng_a
+HomePath = P(tmp) if (tmp := os.getenv("HOMEPATH")) else None  # e.g. C:\Users\eng_a
+Public = P(tmp) if (tmp := os.getenv("PUBLIC")) else None  # C:\Users\Public
 
 
-UserProfile = P(tmp) if (tmp := os.getenv("USERPROFILE")) else None
 OneDriveConsumer = P(tmp) if (tmp := os.getenv("OneDriveConsumer")) else None
 OneDriveCommercial = P(tmp) if (tmp := os.getenv("OneDriveCommercial")) else None
 OneDrive = P(tmp) if (tmp := os.getenv("OneDrive")) else None
 OneDriveExe = LocalAppData.joinpath("Microsoft/OneDrive/OneDrive.exe") if LocalAppData else None
 
 
-def get_shell_profiles(shell):
-    # following this: https://docs.microsoft.com/en-us/powershell/module/microsoft.powershell.core/about/about_profiles?view=powershell-7.2
-    # https://devblogs.microsoft.com/scripting/understanding-the-six-powershell-profiles/
-
-    # Dynmaically obtained:
-    profiles = tb.Struct(
-        CurrentUserCurrentHost = tm.run("$PROFILE.CurrentUserCurrentHost", shell=shell).as_path,
-        CurrentUserAllHosts = tm.run("$PROFILE.CurrentUserAllHosts", shell=shell).as_path,
-        AllUsersCurrentHost = tm.run("$PROFILE.AllUsersCurrentHost", shell=shell).as_path,
-        AllUsersAllHosts = tm.run("$PROFILE.AllUsersAllHosts", shell=shell).as_path,
-    )
-
-    # Static:
-    # profiles = dict(
-    #     Windows = tb.Struct(CurrentUserCurrentHost=tb.P(r"$PSHOME\Profile.ps1"),
-    #                           CurrentUserAllHosts=tb.P(r"$Home\[My ]Documents\PowerShell\Profile.ps1"),
-    #                           AllUsersCurrentHost=tb.P(r"$PSHOME\Microsoft.PowerShell_profile.ps1"),
-    #                           AllUsersAllHosts=tb.P(r"$Home\[My ]Documents\PowerShell\Microsoft.PowerShell_profile.ps1")),
-    # Linux = tb.Struct(CurrentUserCurrentHost=tb.P(r""),
-    #                     CurrentUserAllHosts=tb.P(r""),
-    #                     AllUsersCurrentHost=tb.P(r""),
-    #                     AllUsersAllHosts=tb.P(r"")),
-    # macOS = tb.Struct(CurrentUserCurrentHost=tb.P(r""),
-    #                     CurrentUserAllHosts=tb.P(r""),
-    #                     AllUsersCurrentHost=tb.P(r""),
-    #                     AllUsersAllHosts=tb.P(r"")),
-    # )[system]
-    return profiles
+# ============================== Networking ==============================
 
 
-def construct_path(path_list):
-    from functools import reduce
-    return reduce(lambda x, y: x + sep + y, tb.L(tb.pd.unique(path_list)).apply(str))
+def get_address():
+    netifaces = tb.install_n_import("netifaces")
+    subnet_mask = netifaces.ifaddresses(netifaces.gateways()['default'][netifaces.AF_INET][1])[netifaces.AF_INET][0]['netmask']
+    default_gateway = netifaces.gateways()['default'][netifaces.AF_INET][0]
+
+    import uuid
+    mac = uuid.getnode()
+    mac_address = ":".join(("%012X" % mac)[i:i+2] for i in range(0, 12, 2))
+    # elif hex_format: return hex(mac)
+    # else: return mac
+
+    import socket
+    local_ip_v4 = socket.gethostbyname(socket.gethostname())
+
+    from requests import get
+    public_ip = get('https://api.ipify.org').text
+    return dict(subnet_mask=subnet_mask, mac_address=mac_address, local_ip_v4=local_ip_v4,
+                default_gateway=default_gateway, public_ip=public_ip)
+
+
+# ============================== System Variables ==============================
 
 
 class ShellVar(object):
@@ -188,5 +185,43 @@ class PathVar:
             raise NotImplementedError
 
 
+# ============================== Shells =========================================
+
+
+def get_shell_profiles(shell):
+    # following this: https://docs.microsoft.com/en-us/powershell/module/microsoft.powershell.core/about/about_profiles?view=powershell-7.2
+    # https://devblogs.microsoft.com/scripting/understanding-the-six-powershell-profiles/
+
+    # Dynmaically obtained:
+    profiles = tb.Struct(
+        CurrentUserCurrentHost = tm.run("$PROFILE.CurrentUserCurrentHost", shell=shell).as_path,
+        CurrentUserAllHosts = tm.run("$PROFILE.CurrentUserAllHosts", shell=shell).as_path,
+        AllUsersCurrentHost = tm.run("$PROFILE.AllUsersCurrentHost", shell=shell).as_path,
+        AllUsersAllHosts = tm.run("$PROFILE.AllUsersAllHosts", shell=shell).as_path,
+    )
+
+    # Static:
+    # profiles = dict(
+    #     Windows = tb.Struct(CurrentUserCurrentHost=tb.P(r"$PSHOME\Profile.ps1"),
+    #                           CurrentUserAllHosts=tb.P(r"$Home\[My ]Documents\PowerShell\Profile.ps1"),
+    #                           AllUsersCurrentHost=tb.P(r"$PSHOME\Microsoft.PowerShell_profile.ps1"),
+    #                           AllUsersAllHosts=tb.P(r"$Home\[My ]Documents\PowerShell\Microsoft.PowerShell_profile.ps1")),
+    # Linux = tb.Struct(CurrentUserCurrentHost=tb.P(r""),
+    #                     CurrentUserAllHosts=tb.P(r""),
+    #                     AllUsersCurrentHost=tb.P(r""),
+    #                     AllUsersAllHosts=tb.P(r"")),
+    # macOS = tb.Struct(CurrentUserCurrentHost=tb.P(r""),
+    #                     CurrentUserAllHosts=tb.P(r""),
+    #                     AllUsersCurrentHost=tb.P(r""),
+    #                     AllUsersAllHosts=tb.P(r"")),
+    # )[system]
+    return profiles
+
+
+def construct_path(path_list):
+    from functools import reduce
+    return reduce(lambda x, y: x + sep + y, tb.L(tb.pd.unique(path_list)).apply(str))
+
+
 if __name__ == '__main__':
     pass
