@@ -5,7 +5,7 @@ from crocodile.core import np, List, timestamp, os, Save
 from crocodile.file_management import P
 import typing
 import pandas as pd
-from crocodile.meta import Cycle
+from crocodile.msc.miscellaneous import Cycle
 
 
 class FigurePolicy(enum.Enum):
@@ -87,7 +87,6 @@ class FigureManager:
     def maximize_fig(self):
         """The command required is backend-dependent and also OS dependent."""
         _ = self
-        # plt.get_current_fig_manager().window.state('zoom')
         plt.get_current_fig_manager().full_screen_toggle()
 
     def toggle_annotate(self, event):
@@ -155,8 +154,7 @@ class FigureManager:
         self.message = f'Next {self.index}'
         self.animate()
 
-    def animate(self):
-        pass  # a method of the artist child class that is inheriting from this class
+    def animate(self): pass  # a method of the artist child class that is inheriting from this class
 
     def text_info(self, event):
         _ = event
@@ -256,8 +254,7 @@ class FigureManager:
             fig.canvas.draw()
 
     def update_info_text(self, message):
-        if self.message_obj:
-            self.message_obj.remove()
+        if self.message_obj: self.message_obj.remove()
         self.message_obj = self.fig.text(*self.info_loc, message, fontsize=8)
 
     @staticmethod
@@ -267,12 +264,9 @@ class FigureManager:
             ncols = int(np.ceil(np.sqrt(num_plots)))
             while nrows * ncols < num_plots:
                 ncols += 1
-        elif not ncols and nrows:
-            ncols = int(np.ceil(num_plots / nrows))
-        elif not nrows and ncols:
-            nrows = int(np.ceil(num_plots / ncols))
-        else:
-            pass
+        elif not ncols and nrows: ncols = int(np.ceil(num_plots / nrows))
+        elif not nrows and ncols: nrows = int(np.ceil(num_plots / ncols))
+        else: pass
         return nrows, ncols
 
     def show_cursor(self, event):
@@ -297,69 +291,43 @@ class FigureManager:
                 # event.inaxes.axis(['off', 'on'][self.boundaries_flag])
                 self.toggle_ticks(axis)
                 self.message = f"Boundaries flag set to {self.boundaries_flag} in {axis}"
-
         else:
-            for ax in self.ax:
-                # ax.axis(['off', 'on'][self.boundaries_flag])
-                self.toggle_ticks(ax)
+            for ax in self.ax: self.toggle_ticks(ax)
 
     @staticmethod
     def toggle_ticks(an_ax, state=None):
-        for line in an_ax.get_yticklines():
-            line.set_visible(not line.get_visible() if state is None else state)
-        for line in an_ax.get_xticklines():
-            line.set_visible(not line.get_visible() if state is None else state)
-        for line in an_ax.get_xticklabels():
-            line.set_visible(not line.get_visible() if state is None else state)
-        for line in an_ax.get_yticklabels():
-            line.set_visible(not line.get_visible() if state is None else state)
+        for line in an_ax.get_yticklines(): line.set_visible(not line.get_visible() if state is None else state)
+        for line in an_ax.get_xticklines(): line.set_visible(not line.get_visible() if state is None else state)
+        for line in an_ax.get_xticklabels(): line.set_visible(not line.get_visible() if state is None else state)
+        for line in an_ax.get_yticklabels(): line.set_visible(not line.get_visible() if state is None else state)
 
     def clear_axes(self):
-        for ax in self.ax:
-            ax.cla()
+        for ax in self.ax: ax.cla()
 
     @staticmethod
     def show_pixels_values(ax):
         im = ax.images[0].get_array()
         xmin, xmax = ax.get_xlim()
         ymin, ymax = ax.get_ylim()
-        if ymin > ymax:  # default imshow settings
-            ymin, ymax = ymax, ymin
+        if ymin > ymax: ymin, ymax = ymax, ymin  # default imshow settings
         for (j, i), label in np.ndenumerate(im):
-            if (xmin < i < xmax) and (ymin < j < ymax):
-                ax.text(i, j, np.round(label).__int__(), ha='center', va='center', size=8)
+            if (xmin < i < xmax) and (ymin < j < ymax): ax.text(i, j, np.round(label).__int__(), ha='center', va='center', size=8)
 
     @staticmethod
     def update(figname, obj_name, data=None):
         """Fastest update ever. But, you need access to label path.
         Using this function external to the plotter. But inside the plotter you need to define labels to objects
         The other alternative is to do the update inside the plotter, but it will become very verbose.
-
-        :param figname:
-        :param obj_name:
-        :param data:
-        :return:
         """
-        obj = FigureManager.findobj(figname, obj_name)
-        if data is not None:
-            obj.set_data(data)
-            # update scale:
-            # obj.axes.relim()
-            # obj.axes.autoscale()
+        if data is not None: FigureManager.findobj(figname, obj_name).set_data(data)
 
     @staticmethod
     def findobj(figname, obj_name):
-        if type(figname) is str:
-            fig = plt.figure(num=figname)
-        else:
-            fig = figname
-        search_results = fig.findobj(lambda x: x.get_label() == obj_name)
-        if len(search_results) > 0:  # list of length 1, 2 ...
-            search_results = search_results[0]  # the first one is good enough.
-        return search_results
+        search_results = (plt.figure(num=figname) if type(figname) is str else figname).findobj(lambda x: x.get_label() == obj_name)
+        return search_results[0] if len(search_results) > 0 else search_results
 
-    def get_fig(self, figname='', suffix=None, **kwargs):
-        return FigureManager.get_fig_static(self.figpolicy, figname, suffix, **kwargs)
+    def get_fig(self, figname='', suffix=None, **kwargs): return FigureManager.get_fig_static(self.figpolicy, figname, suffix, **kwargs)
+    def transperent_fig(self): self.fig.canvas.manager.window.attributes("-transparentcolor", "white")
 
     @staticmethod
     def get_fig_static(figpolicy, figname='', suffix=None, **kwargs):
@@ -386,9 +354,6 @@ class FigureManager:
             fig = plt.figure(num=figname, **kwargs)
         return fig
 
-    def transperent_fig(self):
-        self.fig.canvas.manager.window.attributes("-transparentcolor", "white")
-
     @staticmethod
     def set_ax_size(ax, w, h):
         """ w, h: width, height in inches """
@@ -459,11 +424,9 @@ class FigureManager:
         plt.rc('axes', prop_cycle=default_cycler)
         if test:
             temp = np.random.randn(10, 10)
-            for idx, aq in enumerate(temp):
-                plt.plot(aq + idx * 2)
+            for idx, aq in enumerate(temp): plt.plot(aq + idx * 2)
 
-    def close(self):
-        plt.close(self.fig)
+    def close(self): plt.close(self.fig)
 
 
 class SaveType:
@@ -497,23 +460,17 @@ class SaveType:
                 print('Turning off IO')
                 plt.ioff()
 
-            if fignames:  # path sent explicitly
-                self.watch_figs = [plt.figure(figname) for figname in fignames]
+            if fignames: self.watch_figs = [plt.figure(figname) for figname in fignames]  # path sent explicitly
             else:  # tow choices:
                 if self.watch_figs is None:  # None exist ==> add all
                     figure_names = plt.get_figlabels()  # add all.
                     self.watch_figs = [plt.figure(k) for k in figure_names]
-                else:  # they exist already.
-                    pass
+                else: pass  # they exist already.
 
-            if names is None:  # individual save path, useful for PNG.
-                names = [timestamp(name=a_figure.get_label()) for a_figure in self.watch_figs]
+            if names is None: names = [timestamp(name=a_figure.get_label()) for a_figure in self.watch_figs]  # individual save path, useful for PNG.
+            for afig, aname in zip(self.watch_figs, names): self._save(afig, aname, **kwargs)
 
-            for afig, aname in zip(self.watch_figs, names):
-                self._save(afig, aname, **kwargs)
-
-        def _save(self, *args, **kwargs):
-            pass
+        def _save(self, *args, **kwargs): pass
 
     class Null(GenericSave):
         """ Use this when you do not want to save anything. This class will help plot to work faster
@@ -562,8 +519,7 @@ class SaveType:
 
         def _save(self, afigure, aname, dpi=150, **kwargs):
             aname = P(aname).make_valid_filename()
-            afigure.savefig(os.path.join(self.save_dir, aname), bbox_inches='tight', pad_inches=0.3,
-                            dpi=dpi, **kwargs)
+            afigure.savefig(os.path.join(self.save_dir, aname), bbox_inches='tight', pad_inches=0.3, dpi=dpi, **kwargs)
 
         def finish(self):
             print(f"PNGs Saved @", P(self.fname).absolute().as_uri())
@@ -597,7 +553,6 @@ class SaveType:
         def _save(self, afigure, aname, cla=False, **kwargs):
             fig_list = self.container[afigure.get_label()]
             subcontainer = []
-
             search = FigureManager.findobj(afigure, 'neo')
             for item in search:
                 item.set_label('processed')
@@ -644,8 +599,7 @@ class SaveType:
                                     "be setup. Did you mean to use an autosaver?"
             self.writer.setup(fig=self.watch_figs[0], outfile=self.fname, dpi=dpi)
 
-        def _save(self, afig, aname, **kwargs):
-            self.writer.grab_frame(**kwargs)
+        def _save(self, afig, aname, **kwargs): self.writer.grab_frame(**kwargs)
 
         def finish(self):
             print('Saving results ...')
@@ -654,16 +608,13 @@ class SaveType:
             return self.fname
 
     class GIFPipeBased(GIFFileBased):
-        def __init__(self, *args, **kwargs):
-            super().__init__(*args, _type=self.__class__.__name__, **kwargs)
+        def __init__(self, *args, **kwargs): super().__init__(*args, _type=self.__class__.__name__, **kwargs)
 
     class MPEGFileBased(GIFFileBased):
-        def __init__(self, *args, **kwargs):
-            super().__init__(*args, _type=self.__class__.__name__, **kwargs)
+        def __init__(self, *args, **kwargs): super().__init__(*args, _type=self.__class__.__name__, **kwargs)
 
     class MPEGPipeBased(GIFFileBased):
-        def __init__(self, *args, **kwargs):
-            super().__init__(*args, _type=self.__class__.__name__, **kwargs)
+        def __init__(self, *args, **kwargs): super().__init__(*args, _type=self.__class__.__name__, **kwargs)
 
     """
     Parses the data automatically.
@@ -686,8 +637,7 @@ class SaveType:
 
         def animate(self):
             def gen_function():
-                for i in zip(*self.data):
-                    yield i
+                for i in zip(*self.data): yield i
 
             self.data_gen = gen_function
             self.plotter = self.plotter_class(*[piece[0] for piece in self.data], **self.kwargs)
@@ -703,14 +653,11 @@ class SaveType:
             super().__init__(plotter_class, data, **kwargs)
             writer = None
             from matplotlib import animation
-            if extension == 'gif':
-                writer = animation.PillowWriter(fps=fps)
-            elif extension == 'mp4':
-                writer = animation.FFMpegWriter(fps=fps, metadata=dict(artist='Alex Al-Saffar'), bitrate=2500)
+            if extension == 'gif': writer = animation.PillowWriter(fps=fps)
+            elif extension == 'mp4': writer = animation.FFMpegWriter(fps=fps, metadata=dict(artist='Alex Al-Saffar'), bitrate=2500)
 
             def gen_function():
-                for i in zip(*self.data):
-                    yield i
+                for i in zip(*self.data): yield i
 
             self.gen = gen_function
             self.plotter = self.plotter_class(*[piece[0] for piece in self.data], **kwargs)
@@ -770,7 +717,6 @@ class SaveType:
             self.data = gen_function
             self.plotter = plotter_class(*[piece[0] for piece in data], **kwargs)
             plt.pause(0.5)  # give time_produced for figures to show up before updating them
-
             from tqdm import tqdm
             with self.saver.saving(fig=self.plotter.fig, outfile=self.fname, dpi=dpi):
                 for datum in tqdm(self.data()):
@@ -780,16 +726,13 @@ class SaveType:
             print(f"Results saved successfully @ {self.fname}")
 
     class GIFPipeBasedAuto(GIFFileBasedAuto):
-        def __init__(self, *args, **kwargs):
-            super().__init__(*args, _type=self.__class__.__name__, **kwargs)
+        def __init__(self, *args, **kwargs): super().__init__(*args, _type=self.__class__.__name__, **kwargs)
 
     class MPEGFileBasedAuto(GIFFileBasedAuto):
-        def __init__(self, *args, **kwargs):
-            super().__init__(*args, _type=self.__class__.__name__, **kwargs)
+        def __init__(self, *args, **kwargs): super().__init__(*args, _type=self.__class__.__name__, **kwargs)
 
     class MPEGPipeBasedAuto(GIFFileBasedAuto):
-        def __init__(self, *args, **kwargs):
-            super().__init__(*args, _type=self.__class__.__name__, **kwargs)
+        def __init__(self, *args, **kwargs): super().__init__(*args, _type=self.__class__.__name__, **kwargs)
 
 
 class VisibilityViewer(FigureManager):
@@ -857,10 +800,7 @@ class VisibilityViewer(FigureManager):
         if self.fig is None:
             self.fig = artist.fig
             self.fig.canvas.mpl_connect('key_press_event', self.process_key)
-
-        if increment_index:
-            self.index += 1
-            self.index_max += 1
+        if increment_index: self.index += 1; self.index_max += 1
         self.current = self.index
         self.axes_repo.append(self.artist.ax if type(self.artist.ax) is list else self.artist.ax.tolist())
         self.texts_repo.append(self.artist.txt if type(self.artist.txt) is list else self.artist.txt.tolist())
@@ -869,28 +809,22 @@ class VisibilityViewer(FigureManager):
             self.hide_artist_axes()
 
     def hide_artist_axes(self):
-        for ax in self.artist.ax:
-            ax.set_visible(False)
-        for text in self.artist.txt:
-            text.set_visible(False)
+        for ax in self.artist.ax: ax.set_visible(False)
+        for text in self.artist.txt: text.set_visible(False)
+
+    def animate(self):
+        # remove current axes and set self.index as visible.
+        for ax in self.axes_repo[self.current]: ax.set_visible(False)
+        for text in self.texts_repo[self.current]: text.set_visible(False)
+        for ax in self.axes_repo[self.index]: ax.set_visible(True)
+        for text in self.texts_repo[self.index]: text.set_visible(True)
+        self.current = self.index
+        self.fig.canvas.draw()
 
     def finish(self):  # simply: undo the last hiding
         self.current = self.index
         self.animate()
 
-    def animate(self):
-        # remove current axes and set self.index as visible.
-        for ax in self.axes_repo[self.current]:
-            ax.set_visible(False)
-        for text in self.texts_repo[self.current]:
-            text.set_visible(False)
-        for ax in self.axes_repo[self.index]:
-            ax.set_visible(True)
-        for text in self.texts_repo[self.index]:
-            text.set_visible(True)
-        self.current = self.index
-        self.fig.canvas.draw()
-
 
 class VisibilityViewerAuto(VisibilityViewer):
     def __init__(self, data=None, artist=None, memorize=False, transpose=True, save_type=SaveType.Null,
@@ -909,13 +843,10 @@ class VisibilityViewerAuto(VisibilityViewer):
         self.kwargs = kwargs
         self.memorize = memorize
         self.max_index_memorized = 0
-
-        if transpose:
-            data = np.array(list(zip(*data)))
+        if transpose: data = np.array(list(zip(*data)))
         self.data = data
         self.legends = legends
-        if legends is None:
-            self.legends = [f"Curve {i}" for i in range(len(self.data))]
+        if legends is None: self.legends = [f"Curve {i}" for i in range(len(self.data))]
         self.titles = titles if titles is not None else np.arange(len(self.data))
         self.lables = x_labels
 
@@ -924,8 +855,7 @@ class VisibilityViewerAuto(VisibilityViewer):
                             **kwargs)
         else:
             artist.plot(*self.data[0], title=self.titles[0], legends=self.legends)
-            if memorize:
-                assert artist.create_new_axes is True, "Auto Viewer is based on hiding and showing and requires new " \
+            if memorize: assert artist.create_new_axes is True, "Auto Viewer is based on hiding and showing and requires new " \
                                                        "axes from the artist with every plot"
         self.artist = artist
         super().__init__(artist=self.artist, hide_artist_axes=False)
@@ -953,16 +883,13 @@ class VisibilityViewerAuto(VisibilityViewer):
                 self.artist.plot(*datum, title=self.titles[i], legends=self.legends)
                 # replot the new data point on a new axis.
             self.saver.add()
-            if self.pause:
-                break
-            else:
-                self.index = i
+            if self.pause: break
+            else: self.index = i
         if self.index == self.index_max - 1 and not self.pause:  # arrived at last image and not in manual mode
             self.fname = self.saver.finish()
 
     @staticmethod
-    def test():
-        return VisibilityViewerAuto(data=np.random.randn(1, 10, 100, 3))
+    def test(): return VisibilityViewerAuto(data=np.random.randn(1, 10, 100, 3))
 
 
 class ImShow(FigureManager):
@@ -996,7 +923,6 @@ class ImShow(FigureManager):
         Tip: Use np.arrray_split to get sublists and have multiple plots per frame. Useful for very long lists.
         """
         super(ImShow, self).__init__(info_loc=info_loc)
-
         num_plots = len(images_list)  # Number of images in each plot
         self.num_plots = num_plots
         lengths = [len(images_list[i]) for i in range(num_plots)]
@@ -1023,13 +949,9 @@ class ImShow(FigureManager):
         # bprev = Button(axprev, 'Previous')
         # bprev.on_clicked(callback.prev)
 
-        if sup_titles is None:
-            sup_titles = [str(i) for i in np.arange(self.index_max)]
-
-        if labels:
-            sub_labels = [[a_label for _ in np.arange(self.index_max)] for a_label in labels]
-        elif sub_labels is None:
-            sub_labels = [[str(i) for i in np.arange(self.index_max)] for _ in range(self.num_plots)]
+        if sup_titles is None: sup_titles = [str(i) for i in np.arange(self.index_max)]
+        if labels: sub_labels = [[a_label for _ in np.arange(self.index_max)] for a_label in labels]
+        elif sub_labels is None: sub_labels = [[str(i) for i in np.arange(self.index_max)] for _ in range(self.num_plots)]
 
         self.image_list = images_list
         self.sub_labels = sub_labels
@@ -1052,10 +974,8 @@ class ImShow(FigureManager):
             if gridspec is not None:
                 gs = self.fig.add_gridspec(gridspec[0])
                 self.ax = []
-                for ags in gs[1:]:
-                    self.ax.append(self.fig.add_subplot(gs[ags[0], ags[1]]))
-            else:
-                self.ax = self.fig.subplots(nrows=nrows, ncols=ncols)
+                for ags in gs[1:]: self.ax.append(self.fig.add_subplot(gs[ags[0], ags[1]]))
+            else: self.ax = self.fig.subplots(nrows=nrows, ncols=ncols)
         else:
             self.ax = ax
             try:
@@ -1065,10 +985,8 @@ class ImShow(FigureManager):
 
         self.fig.canvas.mpl_connect('key_press_event', self.process_key)
         self.fig.canvas.mpl_connect("pick_event", self.annotate)
-        if tight:
-            self.fig.tight_layout()
-        if subplots_adjust is not None:
-            self.fig.subplots_adjust(**subplots_adjust)
+        if tight: self.fig.tight_layout()
+        if subplots_adjust is not None: self.fig.subplots_adjust(**subplots_adjust)
 
         # if save_type.parser == "internal":
         #     raise TypeError("Requires external data parser")
@@ -1078,8 +996,7 @@ class ImShow(FigureManager):
                                delay=delay, fps=1000 / delay, **save_kwargs)
         if nrows == 1 and ncols == 1:
             self.ax = [self.ax]  # make a list out of it.
-        else:
-            self.ax = self.ax.ravel()  # make a 1D  list out of a 2D array.
+        else: self.ax = self.ax.ravel()  # make a 1D  list out of a 2D array.
         for an_ax in self.ax:
             # an_ax.set_xticks([])
             # an_ax.set_yticks([])
@@ -1101,10 +1018,8 @@ class ImShow(FigureManager):
                 if i == 0 and self.ims.__len__() < self.num_plots:
                     im = an_ax.imshow(an_image, animated=True, **self.kwargs)
                     self.ims.append(im)
-                else:
-                    self.ims[j].set_data(an_image)
-                if self.auto_brightness:
-                    self.ims[j].norm.autoscale(an_image)
+                else: self.ims[j].set_data(an_image)
+                if self.auto_brightness: self.ims[j].norm.autoscale(an_image)
                 an_ax.set_xlabel(f'{a_label}')
             self.fig.suptitle(self.titles[i], fontsize=8)
             self.saver.add(names=[self.titles[i]])
@@ -1116,8 +1031,7 @@ class ImShow(FigureManager):
             self.fname = self.saver.finish()
 
     def annotate(self, event, axis=None, data=None):
-        for ax in self.ax:
-            super().annotate(event, axis=ax, data=ax.images[0].get_array())
+        for ax in self.ax: super().annotate(event, axis=ax, data=ax.images[0].get_array())
 
     @classmethod
     def from_saved_images_path_lists(cls, *image_list, **kwargs):
@@ -1136,17 +1050,14 @@ class ImShow(FigureManager):
     @classmethod
     def from_directories(cls, *directories, extension='png', **kwargs):
         paths = []
-        for a_dir in directories:
-            paths.append(P(a_dir).search(f"*.{extension}", win_order=True))
+        for a_dir in directories: paths.append(P(a_dir).search(f"*.{extension}", win_order=True))
         return cls.from_saved_images_path_lists(*paths, **kwargs)
 
     @classmethod
     def from_saved(cls, *things, **kwargs):
         example_item = things[0]
-        if isinstance(example_item, list):
-            return cls.from_saved_images_path_lists(*things, **kwargs)
-        else:
-            return cls.from_directories(*things, **kwargs)
+        if isinstance(example_item, list): return cls.from_saved_images_path_lists(*things, **kwargs)
+        else: return cls.from_directories(*things, **kwargs)
 
     @staticmethod
     def cm(im, nrows=3, ncols=7, **kwargs):
@@ -1172,8 +1083,7 @@ class ImShow(FigureManager):
         pass
 
     @staticmethod
-    def test():
-        return ImShow(*np.random.randn(12, 101, 100, 100))
+    def test(): return ImShow(*np.random.randn(12, 101, 100, 100))
 
     @classmethod
     def complex(cls, data, pause=True, **kwargs):
@@ -1224,18 +1134,11 @@ class Artist(FigureManager):
 
     def accessorize(self, *args, legends=None, title=None, **kwargs):
         self.line = self.ax[0].plot(*args, **kwargs)
-        if legends is not None:
-            self.ax[0].legend(legends)
-        if title is not None:
-            self.ax[0].set_title(title)
+        if legends is not None: self.ax[0].legend(legends)
+        if title is not None: self.ax[0].set_title(title)
         self.ax[0].grid('on')
 
-    def get_axes(self):
-        if self.create_new_axes:
-            axis = self.fig.subplots()
-            self.ax = np.array([axis])
-        else:  # use same old axes
-            pass
+    def get_axes(self): self.ax = np.array([self.fig.subplots()]) if self.create_new_axes else self.ax
 
     def suptitle(self, title):
         self.txt = [self.fig.text(0.5, 0.98, title, ha='center', va='center', size=9)]
